package solutions.bitbadger.documents.common

import solutions.bitbadger.documents.*
import solutions.bitbadger.documents.query.Count
import solutions.bitbadger.documents.query.Find
import java.sql.Connection
import kotlin.test.assertEquals
import kotlin.test.assertNotNull
import kotlin.test.assertNull

/**
 * Integration tests for the `Custom` object
 */
object Custom {

    fun listEmpty(conn: Connection) {
        JsonDocument.load(conn, TEST_TABLE)
        conn.customNonQuery("DELETE FROM $TEST_TABLE")
        val result = conn.customList<JsonDocument>(Find.all(TEST_TABLE), mapFunc = Results::fromData)
        assertEquals(0, result.size, "There should have been no results")
    }

    fun listAll(conn: Connection) {
        JsonDocument.load(conn, TEST_TABLE)
        val result = conn.customList<JsonDocument>(Find.all(TEST_TABLE), mapFunc = Results::fromData)
        assertEquals(5, result.size, "There should have been 5 results")
    }

    fun singleNone(conn: Connection) =
        assertNull(
            conn.customSingle(Find.all(TEST_TABLE), mapFunc = Results::fromData),
            "There should not have been a document returned"
        )

    fun singleOne(conn: Connection) {
        JsonDocument.load(conn, TEST_TABLE)
        assertNotNull(
            conn.customSingle<JsonDocument>(Find.all(TEST_TABLE), mapFunc = Results::fromData),
            "There should not have been a document returned"
        )
    }

    fun nonQueryChanges(conn: Connection) {
        JsonDocument.load(conn, TEST_TABLE)
        assertEquals(
            5L, conn.customScalar(Count.all(TEST_TABLE), mapFunc = Results::toCount),
            "There should have been 5 documents in the table"
        )
        conn.customNonQuery("DELETE FROM $TEST_TABLE")
        assertEquals(
            0L, conn.customScalar(Count.all(TEST_TABLE), mapFunc = Results::toCount),
            "There should have been no documents in the table"
        )
    }

    fun nonQueryNoChanges(conn: Connection) {
        JsonDocument.load(conn, TEST_TABLE)
        assertEquals(
            5L, conn.customScalar(Count.all(TEST_TABLE), mapFunc = Results::toCount),
            "There should have been 5 documents in the table"
        )
        conn.customNonQuery(
            "DELETE FROM $TEST_TABLE WHERE data->>'id' = :id",
            listOf(Parameter(":id", ParameterType.STRING, "eighty-two"))
        )
        assertEquals(
            5L, conn.customScalar(Count.all(TEST_TABLE), mapFunc = Results::toCount),
            "There should still have been 5 documents in the table"
        )
    }

    fun scalar(conn: Connection) {
        JsonDocument.load(conn, TEST_TABLE)
        assertEquals(
            3L,
            conn.customScalar("SELECT 3 AS it FROM $TEST_TABLE LIMIT 1", mapFunc = Results::toCount),
            "The number 3 should have been returned"
        )
    }
}
