package solutions.bitbadger.documents.postgresql

import solutions.bitbadger.documents.AutoId
import solutions.bitbadger.documents.Configuration
import solutions.bitbadger.documents.customNonQuery
import solutions.bitbadger.documents.ensureTable

/**
 * A wrapper for a throwaway PostgreSQL database
 */
class PgDB : AutoCloseable {

    private var dbName = ""

    init {
        dbName = "throwaway_${AutoId.generateRandomString(8)}"
        Configuration.connectionString = connString("postgres")
        Configuration.dbConn().use {
            it.customNonQuery("CREATE DATABASE $dbName")
        }
        Configuration.connectionString = connString(dbName)
    }

    val conn = Configuration.dbConn()

    init {
        conn.ensureTable(tableName)
    }

    override fun close() {
        conn.close()
        Configuration.connectionString = connString("postgres")
        Configuration.dbConn().use {
            it.customNonQuery("DROP DATABASE $dbName")
        }
        Configuration.connectionString = null
    }

    companion object {

        /** The table used for test documents */
        val tableName = "test_table"

        /**
         * Create a connection string for the given database
         *
         * @param database The database to which the library should connect
         * @return The connection string for the database
         */
        private fun connString(database: String) =
            "jdbc:postgresql://localhost/$database?user=postgres&password=postgres"
    }
}
