package solutions.bitbadger.documents.sqlite

import org.junit.jupiter.api.DisplayName
import solutions.bitbadger.documents.*
import java.sql.Connection
import kotlin.test.Test
import kotlin.test.assertFalse
import kotlin.test.assertTrue

/**
 * SQLite integration tests for the `Definition` object / `ensure*` connection extension functions
 */
@DisplayName("SQLite - Definition")
class DefinitionIT {

    /**
     * Determine if a database item exists
     *
     * @param item The items whose existence should be checked
     * @param conn The current database connection
     * @return True if the item exists in the given database, false if not
     */
    private fun itExists(item: String, conn: Connection) =
        conn.customScalar("SELECT EXISTS (SELECT 1 FROM ${SQLiteDB.CATALOG} WHERE name = :name) AS it",
            listOf(Parameter(":name", ParameterType.STRING, item)), Results::toExists)

    @Test
    @DisplayName("ensureTable creates table and index")
    fun ensureTable() =
        SQLiteDB().use { db ->
            assertFalse(itExists("ensured", db.conn), "The 'ensured' table should not exist")
            assertFalse(itExists("idx_ensured_key", db.conn), "The PK index for the 'ensured' table should not exist")
            db.conn.ensureTable("ensured")
            assertTrue(itExists("ensured", db.conn), "The 'ensured' table should exist")
            assertTrue(itExists("idx_ensured_key", db.conn), "The PK index for the 'ensured' table should now exist")
        }

    @Test
    @DisplayName("ensureFieldIndex creates an index")
    fun ensureFieldIndex() =
        SQLiteDB().use { db ->
            assertFalse(itExists("idx_${TEST_TABLE}_test", db.conn), "The test index should not exist")
            db.conn.ensureFieldIndex(TEST_TABLE, "test", listOf("id", "category"))
            assertTrue(itExists("idx_${TEST_TABLE}_test", db.conn), "The test index should now exist")
        }
}
